#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>

#include <dbui-defs.h>

#include <dbui-structs.h>

#include <dbui-sql-proto.h>

static char *
get_equal_col_eq_str(COLUMN *col,char *where_value);

#define LIMIT_START " LIMIT %s "
#define ORDER_START " ORDER BY %s "

char *
get_limit_str(TABLE	*table)
{
    char	*return_str;
    int		return_str_len;

    if(!table || !table->limit_str)
	return NULL; 

    return_str_len = strlen(LIMIT_START) + strlen(table->limit_str);

    return_str = (char *)malloc(return_str_len * sizeof(char));

    sprintf(return_str,LIMIT_START,table->limit_str);
      
    return return_str; 
}

char *
get_order_by_str(TABLE *table)
{
    GList	*g_l;
    COLUMN	*col;
    char	*return_str = NULL;
    char	*order_str = NULL;
    char	**list;
    int		list_len = 0;
    int		list_str_len = 0;
    int		i = 0;
    int		x = 0;

    if(!table || !table->current_value)
	return NULL;

    g_l = table->current_value;

    list_len = g_list_length(g_l);

    list = (char **)malloc(list_len * sizeof(char *));
    while(g_l)
	{
	list[x] = NULL;
	x++;	
	if(g_l->data)
	  {
	  col = g_l->data;
	  if(col->order_by)
	    {
	    list[i] = get_col_full_name(col);
	    list_str_len += strlen(list[i]);
	    i++;
	    }
	  }
	g_l = g_list_next(g_l);
	}

    if(i)
	{
	order_str = get_con_cat_list(list," , ",i,list_str_len);

	if(order_str)
	  {
	  return_str = (char *)calloc(strlen(order_str) + strlen(ORDER_START) 
			+ 1, sizeof(char ));
	  sprintf(return_str,ORDER_START,order_str);

	  free(order_str);
	  }
	x = 0;
	while(list[x])
	  {
	  free(list[x]);
	  x++;
	  }
	}

    free(list);

    return return_str;
}

char *
get_col_where_clause(TABLE	*table)
{
    GList       *col_set;
    COLUMN      *col;
    int		i;
    int         where_len = 0;
    int         num_where = 0;
    int         num_fields = 0;
    int         total_len = 0;
    char	*equal_type = NULL;
    char        **where_clause;
    char        *where_str;
    char	*str = NULL;
    char	*equal_str;

    if(!table || !table->col_set )
        return NULL;

    if(table->where_value && table->use_where_value)
	col_set = table->where_value;
    else
	col_set = table->current_value;

    num_fields = g_list_length(col_set);

    where_clause = (char **)malloc(num_fields * sizeof(char *));

    if(!where_clause)
	return NULL;

    for(i = 0 ; i < num_fields && col_set; i++)
        {
          if(col_set->data)
          {
          col = (COLUMN *)col_set->data;

	  switch(col->compair_type){
		case DBUI_LIKE:
			equal_type = DBUI_LIKE_STRING;
			break;
		case DBUI_EQUAL:
			equal_type = DBUI_EQUAL_STRING;
			break;
		case DBUI_GRT:
			equal_type = DBUI_GRT_STRING;
			break;
		case DBUI_LT:
			equal_type = DBUI_LT_STRING;
			break;
	 	default:
			equal_type = DBUI_EQUAL_STRING;
                        break;
	  }

          where_clause[i] = get_col_equal_string(col,equal_type,TRUE);

          if(col->equal_col)
            {
            where_clause[i] = get_equal_col_eq_str(col,where_clause[i]);
            }


          if(where_clause[i])
                {
                num_where++;
                where_len += strlen(where_clause[i]);
                }

	  }
	col_set = g_list_next(col_set);
	}

    if(num_where)
	{
	where_str = get_con_cat_list(where_clause," and ",num_fields,where_len);
	if(where_str)
	  {
	  total_len = strlen(" WHERE ") + strlen(where_str) + 2;
	  str = (char *)calloc(total_len , sizeof(char)); 
	  snprintf(str , total_len," WHERE %s ",where_str);
	  free(where_str);
	  }
	}

    for(i = 0 ; i < num_fields ; i ++)
	{
	if(where_clause[i])
	  free(where_clause[i]);
	}

    free(where_clause);

    return str;    
}

static char *
get_equal_col_eq_str(COLUMN *col,char *where_value)
{
    char	*col_full_name;
    char	*eq_col_full_name;
    char	*return_str;
    char	*eq_str;
    char	*list[2];
    int		list_len = 0;
    int		eq_len = 0;
    int		where_len = 0;


    if(!col || !col->equal_col)
	return where_value;

    col_full_name = get_col_full_name(col);
    eq_col_full_name = get_col_full_name(col->equal_col);

    list[0] = col_full_name;
    list[1] = eq_col_full_name;

    if(col_full_name)
	list_len = strlen(col_full_name);

    if(eq_col_full_name)
	list_len += strlen(eq_col_full_name);

    eq_str =  get_con_cat_list(list," = ",2,list_len);

    if(eq_str)
	{
	if(where_value)
	  {
	  where_len = strlen(where_value);
	  eq_len = strlen(eq_str);
	  return_str = (char *)calloc(where_len + eq_len + 1 + strlen(" and "),
					sizeof(char));

	  sprintf(return_str,"%s and %s",where_value,eq_str);
	  free(where_value);
	  free(eq_str);
	  }
	else
	  return_str = eq_str;
	}
     else
	return_str = where_value;

    return return_str;
}

char *
get_col_table_names(TABLE *table)
{
    GList       *col_set;
    COLUMN      *col;
    char        *table_str;
    char	*table_name;
    char        **tables;
    int         num_fields = 0;
    int		num_table_names = 0;
    int         total_len = 0;
    int		full_length = 0;
    int		i,j;

    if(!table || !table->col_set )
        return NULL;

    col_set = table->col_set;

    num_fields = g_list_length(col_set);

    tables = (char **)malloc((num_fields * 2) * sizeof(char *));

    if(!tables)
        return NULL; 

    for(i = 0 ; i < num_fields && col_set; i++)
	{
	  tables[i] = NULL;
	  tables[i + 1] = NULL;

	  if(col_set->data)
	  {
	  col = (COLUMN *)col_set->data;

	  table_name = get_col_table_name(col);

	  if(!table_name)
	    {
	    i = num_fields;
            continue;
	    }
	  full_length += 1;

	  for(j = 0 ; tables[j] && j < i * 2; j++)
	    {
	    if(!strncmp(tables[j],table_name,NAME_LENGTH) )
		{
		free(table_name);
		table_name = NULL;
		j = i;
		}
	    }
	  if(table_name)
		{
		tables[j] = table_name;
		total_len += strlen(table_name);
		}

	   if(col->equal_col)
		{
		table_name = get_col_table_name(col->equal_col);

		if(!table_name)
	            	{
        		i = num_fields;
            		continue;
            		}

		for(j = 0 ; tables[j] && j < i * 2; j++)
			{
		  	if(!strncmp(tables[j],table_name,NAME_LENGTH) )
			  {
                	  free(table_name);
                	  table_name = NULL;
                	  j = i;
                	  }
            		}
          	if(table_name)
                	{
                	tables[j] = table_name;
                	total_len += strlen(table_name);
                	}

		}
	  }	

	col_set = g_list_next(col_set);
	}

    if(full_length == num_fields)
	{
	table_str = get_con_cat_list(tables," , ",num_fields,total_len); 
	}

    for(i = 0 ; i < num_fields ; i++)
	{
	if(tables[i])
	  free(tables[i]);
	}

    free(tables);

    return table_str;
}


char *
get_col_full_names_list(TABLE *table)
{
    GList       *col_set;
    COLUMN      *col;
    int         i;
    int         num_fields = 0;
    int		full_length = 0;
    int         total_len = 0;
    char        *str = NULL;
    char	**col_names;

    if(!table || !table->col_set)
	return NULL;

    col_set = table->col_set;

    num_fields = g_list_length(col_set);

    if(num_fields == 0 )
	return NULL;

    col_names = (char **)malloc(num_fields * sizeof(char *));

    for(i = 0 ; col_set && i < num_fields ; i++)
	{
	if(!col_set->data)
	  {
	  i = num_fields;
	  continue;
	  }
	col = col_set->data;
	col_names[i] = get_col_full_name(col);

	if(!col_names[i])
	   {
	   i = num_fields;
	   continue;
	   }
	total_len += strlen(col_names[i]);
	full_length += 1;
	col_set = g_list_next(col_set);
	}

    if(full_length == num_fields)
	{
	str = get_con_cat_list(col_names," , ",num_fields,total_len);
	}

    for(i = 0 ; i < num_fields ; i++)
	{
	if(col_names[i])
	  free(col_names[i]);
	}

    free(col_names);
    return str;
}

char *
get_col_value(COLUMN    *col)
{
    int         value_len = 0;
    char        *value;

    if(!col || !col->value )
        return NULL;

    value_len = strlen(col->value);

    if(value_len < 1)
        return NULL;

    if(col->type > 7 && col->need_quote)
        {
        value_len += 2;
        }

    value = (char *)calloc((value_len + 1) , sizeof(char));

    if(!value)
        return NULL;

    if(col->type > 7 && col->need_quote)
        {
        value[0] = '\'';
        value[value_len - 1] = '\'';

        memcpy(value + 1, col->value,value_len - 2);
        }
    else
        strncpy(value , col->value , value_len);

    return value;
}

char *
get_col_table_name(COLUMN       *col)
{
    TABLE       *table;
    int         name_len = 0;
    char        *table_name;

    if(!col || col->parent_name[0] == '\0')
        return NULL;

    return strdup(col->parent_name);
}

char *
get_col_full_name(COLUMN        *col)
{
    char        *full_name;
    char        *table_name;
    int         str_len,str_len2;
    int         total_len;

    if(!col || !col->table )
        return NULL;

    table_name = col->parent_name;

    str_len2 = strlen(col->name);

    total_len = str_len2;

    str_len = strlen(table_name);

    total_len += str_len + 2;

    full_name = (char *)calloc(total_len ,sizeof(char));

    memcpy(full_name,table_name,str_len);

    full_name[str_len] = '.';

    memcpy(full_name + str_len + 1, col->name , str_len2);

    return full_name;
}

char *
get_con_cat_list(char   **list,char *delimiter,int num_entries,int total_len)
{
    char        *str = NULL;
    int         i;
    int         delimit_len = 0;
    int         l_len;
    int         pos = 0;
    int         str_total_len = 0;

    delimit_len = strlen(delimiter);

    str_total_len = (total_len + (delimit_len * num_entries + 1));

    str = (char *)calloc( str_total_len , sizeof(char));

    if(!str)
        return NULL;

    for(i = 0 ; i < num_entries ; i++)
        {
        if(list[i])
          {
          l_len = strlen(list[i]);
	  if(pos)
	    { 	
	    memcpy(str + pos ,delimiter,delimit_len);
	    pos += delimit_len;	
	    }

          memcpy(str + pos , list[i],l_len);
          pos += l_len;

          }
        }

    return str;
}

char * get_col_equal_type_string(COLUMN *col)
{
    char *equal_type;

    switch(col->compair_type){
           case DBUI_LIKE:
                equal_type = DBUI_LIKE_STRING;
                break;
           case DBUI_EQUAL:
                equal_type = DBUI_EQUAL_STRING;
                break;
           case DBUI_GRT:
                equal_type = DBUI_GRT_STRING;
                break;
           case DBUI_LT:
                equal_type = DBUI_LT_STRING;
                break;
           default:
                equal_type = DBUI_EQUAL_STRING;
                break;
    } 

    return get_col_equal_string(col,equal_type,TRUE);
}

char *get_col_equal_string(COLUMN *col,char *type,int need_full_name)
{
    char        *str;
    char        *full_name;
    char        *value;
    int         name_len;
    int         value_len;
    int         type_len;
    int         total_len;

    if(!col || !type)
        return NULL;

    type_len = strlen(type);

    if(need_full_name == TRUE)
    	full_name = get_col_full_name(col);
    else
	full_name = strdup(col->name);

    if(!full_name)
        return NULL;

    name_len = strlen(full_name);

    value = get_col_value(col);

    if(!value)
        {
        free(full_name);
        return NULL;
        }

    value_len = strlen(value );

    total_len = type_len + name_len + value_len  + 3;

    str = (char *)calloc(total_len , sizeof(char));

    if(!str)
        {
        free(value);
        free(full_name);
        return NULL;
        }
    memcpy(str,full_name,name_len);

    str[name_len] = ' ';

    memcpy(str + name_len + 1,type,type_len);

    str[name_len + type_len + 1] = ' ';

    memcpy(str + name_len + type_len + 2 , value , value_len);

    free(full_name);
    free(value);

    return str;
}

